#!/usr/bin/perl

use Libconf::Glueconf::Generic::Shell;
use Data::Dumper;
use Libconf::Test;
$Libconf::Test::test_module_name = 'shell';

# test 1, minimal requirement : regenerate correctly
Libconf::Test::display(1);
my $network = new Libconf::Glueconf::Generic::Shell({ filename => 'ifcfg-eth0' });
$network->write_conf('shell_output_1');
Libconf::Test::test(1);

# test 2, modifying some infos
Libconf::Test::display(2);
$network->{DEVICE} = 'eth1';
$network->write_conf('shell_output_2');
Libconf::Test::test(2);

### same with proxy
$Libconf::Strict = 0;
# test 1, minimal requirement : regenerate correctly
Libconf::Test::display(3);
my $proxy = new Libconf::Glueconf::Generic::Shell({ filename => 'proxy.sh',
                                                    shell_style => 'sh',
                                                    export_line => 1,
                                                    export_only_defined => 0,
                                                  });
$proxy->write_conf('shell_output_3');
Libconf::Test::test(3);

# test 2, modifying some infos
Libconf::Test::display(4);
$proxy->{http_proxy} = 'test.com';
$proxy->write_conf('shell_output_4');
Libconf::Test::test(4);

# test 5, make.conf
Libconf::Test::display(5);
my $make_conf = new Libconf::Glueconf::Generic::Shell({ filename => "make.conf" });
$make_conf->write_conf('shell_output_5');
Libconf::Test::test(5);

# test 6, make.conf, simplify_quote
Libconf::Test::display(6);
my $make_conf = new Libconf::Glueconf::Generic::Shell({ filename => "make.conf",
                                                        simplify_quote => 1,
                                                      });
$make_conf->write_conf('shell_output_6');
Libconf::Test::test(6);

# test 7, make.conf, true_shell
Libconf::Test::display(7);
my $make_conf = new Libconf::Glueconf::Generic::Shell({ filename => "make.conf",
                                                        shell_style => "true_bash",
                                                        shell_command => "/bin/bash",
                                                      });
$make_conf->{CHOST} = 'TEST"BAR';
$make_conf->write_conf('shell_output_7');
Libconf::Test::test(7);

# test 8, make.conf, true_shell, comments
Libconf::Test::display(8);
my $make_conf = new Libconf::Glueconf::Generic::Shell({ filename => "make.conf",
                                                        shell_style => "true_bash",
                                                        shell_command => "/bin/bash",
                                                      });
$make_conf->get_comments('{CHOST}')->{normal} eq '
# Host Setting
# ============
#
# All PowerPC systems should use this host setting:
' or Libconf::Test::failed(8);


Libconf::Test::display(-1);
