#!/usr/bin/perl

# Author : Damien KROTKINE (damien@tuxfamily.org)
#
# Contributors : 
#
# Copyright (C) 2002 Damien KROTKINE (damien@tuxfamily.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package Libconf::Test;
use strict;
use vars qw(@ISA @EXPORT);
our $test_module_name;
my $OUTPUT_COLORS;
$ENV{OUTPUT_COLORS} and $OUTPUT_COLORS=$ENV{OUTPUT_COLORS};
defined $OUTPUT_COLORS or $OUTPUT_COLORS = (`tput colors` >= 8);

sub display {
    local $| = 1;
    $_[0] == -1 and print ' ' . green('done') . "\n";
    $_[0] == 1 and print "   " . yellow('*') . " testing $test_module_name " . '.';
    $_[0] > 1 and print '.';
    local $| = 0;
}

sub skip {
    local $| = 1;
    $_[0] == 1 and print "   " . yellow('*') . " testing $test_module_name " . yellow('S');
    $_[0] > 1 and print yellow('_');
    local $| = 0;
}

sub test {
    my ($num, $exec) = @_;
    $exec ||= "diff -q " . $test_module_name . "_output_$num " . $test_module_name . "_verify_$num 2>/dev/null 1>/dev/null";
    if (system($exec) != 0) {
        failed($num);
    }
}

sub failed {
    my ($num) = @_;
    print "\r   " . yellow('*') . " testing $test_module_name : TEST N$num " . red('FAILED') . "\n";
    exit($!);
}

sub red {
    my ($str) = @_;
    $OUTPUT_COLORS or return $str;
    return "\033[1m\033[31m$str\033[0m";
}

sub green {
    my ($str) = @_;
    $OUTPUT_COLORS or return $str;
    return "\033[1m\033[32m$str\033[0m";
}

sub yellow {
    my ($str) = @_;
    $OUTPUT_COLORS or return $str;
    return "\033[1m\033[33m$str\033[0m";
}

1;
