#!/usr/bin/perl

use Libconf qw(:functions);
use Libconf::Test;
$Libconf::Test::test_module_name = 'smb.conf';

use Libconf::Templates::Samba;

# test 1, minimal requirement : regenerate correctly
Libconf::Test::display(1);
my $template = new Libconf::Templates::Samba({filename => 'smb.conf'});
$template->read_conf();
$template->write_conf('smb.conf_output_1');
Libconf::Test::test(1);

# test 2, editing an atom
Libconf::Test::display(2);
$template = new Libconf::Templates::Samba({filename => 'smb.conf'});
$template->read_conf();
$template->edit_atom(-1, {key => 'writable', sections => [{name => 'homes'}]}, {value => 'TEST' });
$template->write_conf("smb.conf_output_2");
Libconf::Test::test(2);

# test 3, appending an atom
Libconf::Test::display(3);
$template = new Libconf::Templates::Samba({filename => 'smb.conf'});
$template->read_conf();
$template->append_atom({key => 'K1', value => 'yo', type => 'KEY_VALUE', sections => [ { name => 'partage' } ]});
$template->write_conf("smb.conf_output_3");
Libconf::Test::test(3);

# test 4, inserting a atom
Libconf::Test::display(4);
$template = new Libconf::Templates::Samba({filename => 'smb.conf'});
$template->read_conf();
$template->insert_atom(
                       $template->find_atom_pos({type => 'KEY_VALUE', sections => [ { name => 'homes' } ] })+1,
                       {key => 'TESTkey', value => 'TESTvalue', type => 'KEY_VALUE' }
                      );
$template->write_conf("smb.conf_output_4");
Libconf::Test::test(4);

# test 5, getting an atom
Libconf::Test::display(5);
$template = new Libconf::Templates::Samba({filename => 'smb.conf'});
$template->read_conf();
my $atom = $template->get_atom(1);
$atom->{value} eq 'MDKGROUP' or Libconf::Test::failed(5);
$atom->{key} eq 'workgroup' or Libconf::Test::failed(5);
$atom->{sections}->[0]->{name} eq 'global' or Libconf::Test::failed(5);

# test 6, test for stable loop
Libconf::Test::display(6);
$template = new Libconf::Templates::Samba({filename => 'smb.conf'});
$template->read_conf();
$template->write_conf("smb.conf_output_6");
$template = new Libconf::Templates::Samba({filename => 'smb.conf_output_6'});
$template->read_conf();
$template->write_conf("smb.conf_output_6");
Libconf::Test::test(6);

Libconf::Test::display(-1);
