#!/usr/bin/perl

use Libconf qw(:functions);
use Data::Dumper;
use Libconf::Test;
$Libconf::Test::test_module_name = 'autologin';

use Libconf::Templates::Shell;

# test 1, minimal requirement : regenerate correctly
Libconf::Test::display(1);
my $template = new Libconf::Templates::Shell({filename => 'autologin'});
$template->read_conf();
$template->write_conf("autologin_output_1");
Libconf::Test::test(1);

# test 2, editing an atom
Libconf::Test::display(2);
$template = new Libconf::Templates::Shell({filename => 'autologin', simplify_quote => 1}, );
$template->read_conf();
$template->edit_atom(-1, {key => 'USER'}, {value => 'plop'});
$template->edit_atom(-1, {key => 'AUTOLOGIN'}, {value => 'no'});
$template->edit_atom(-1, {key => 'EXEC'}, {value => 'startx'});
$template->write_conf("autologin_output_2");
Libconf::Test::test(2);


# test 3, appending an atom
Libconf::Test::display(3);
$template = new Libconf::Templates::Shell({filename => 'autologin'});
$template->read_conf();
$template->append_atom({key => 'K1', value => 'yo', type => 'KEY_VALUE'});
$template->write_conf("autologin_output_3");
Libconf::Test::test(3);

# test 4, inserting a atom
Libconf::Test::display(4);
$template = new Libconf::Templates::Shell({filename => 'autologin'});
$template->read_conf();
$template->insert_atom(1, {key => 'K2', value => 'yo2', type => 'KEY_VALUE'});
$template->write_conf("autologin_output_4");
Libconf::Test::test(4);

# test 5, getting an atom
Libconf::Test::display(5);
$template = new Libconf::Templates::Shell({filename => 'autologin'});
$template->read_conf();
my $atom = $template->get_atom(1);
$atom->{value} eq 'yes or no, depends' or Libconf::Test::failed(5);
$atom->{type} eq 'KEY_VALUE' or Libconf::Test::failed(5);
$atom->{key} eq 'AUTOLOGIN' or Libconf::Test::failed(5);

# test 6, searching for an atom
Libconf::Test::display(6);
$template = new Libconf::Templates::Shell({filename => 'autologin'});
$template->read_conf();
$atom = $template->find_atom_pos({key => 'AUTOLOGIN', value => 'yes or no, depends'});
Libconf::output("autologin_output_6", $atom . "\n");
Libconf::Test::test(6);

# test 7, test for stable loop
Libconf::Test::display(7);
$template = new Libconf::Templates::Shell({filename => 'autologin'});
$template->read_conf();
$template->write_conf("autologin_output_7");
$template = new Libconf::Templates::Shell({filename => 'autologin_output_7'});
$template->read_conf();
$template->write_conf("autologin_output_7");
Libconf::Test::test(7);

Libconf::Test::display(-1);
